// Generated by CoffeeScript 1.8.0
var chmod, chown, execute, misc, wrap;

module.exports = function(goptions, options, callback) {
  return wrap(arguments, function(options, next) {
    var do_chmod, do_chown, do_end, do_get, do_ktadd, do_ktremove, modified;
    if (!options.principal) {
      return next(new Error('Property principal is required'));
    }
    if (!options.keytab) {
      return next(new Error('Property keytab is required'));
    }
    if (/.*@.*/.test(options.kadmin_principal)) {
      if (options.realm == null) {
        options.realm = options.kadmin_principal.split('@')[1];
      }
    }
    modified = false;
    do_get = function() {
      if (!options.keytab) {
        return do_end();
      }
      return execute({
        cmd: "klist -kt " + options.keytab,
        ssh: options.ssh,
        log: options.log,
        stdout: options.stdout,
        stderr: options.stderr,
        code_skipped: 1
      }, function(err, exists, stdout, stderr) {
        var keytab, kvno, line, match, mdate, principal, _, _i, _len, _ref;
        if (err) {
          return next(err);
        }
        if (!exists) {
          return do_ktadd();
        }
        keytab = {};
        _ref = stdout.split('\n');
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          line = _ref[_i];
          if (match = /^\s*(\d+)\s+([\d\/:]+\s+[\d\/:]+)\s+(.*)\s*$/.exec(line)) {
            _ = match[0], kvno = match[1], mdate = match[2], principal = match[3];
            kvno = parseInt(kvno, 10);
            mdate = Date.parse("" + mdate + " GMT");
            if (!keytab[principal] || keytab[principal].kvno < kvno) {
              keytab[principal] = {
                kvno: kvno,
                mdate: mdate
              };
            }
          }
        }
        if (keytab[options.principal] == null) {
          return do_ktadd();
        }
        return execute({
          cmd: misc.kadmin(options, "getprinc -terse " + options.principal),
          ssh: options.ssh,
          log: options.log,
          stdout: options.stdout,
          stderr: options.stderr
        }, function(err, exists, stdout, stderr) {
          var values, _ref1, _ref2, _ref3;
          if (err) {
            return err;
          }
          values = stdout.split('\n')[1];
          if (!values) {
            return next(Error("Principal does not exist: '" + options.principal + "'"));
          }
          values = values.split('\t');
          mdate = parseInt(values[2], 10) * 1000;
          kvno = parseInt(values[8], 10);
          if (typeof options.log === "function") {
            options.log("Mecano `krb5_ktadd`: keytab kvno '" + ((_ref1 = keytab[principal]) != null ? _ref1.kvno : void 0) + "', principal kvno '" + kvno + "'");
          }
          if (typeof options.log === "function") {
            options.log("Mecano `krb5_ktadd`: keytab mdate '" + (new Date((_ref2 = keytab[principal]) != null ? _ref2.mdate : void 0)) + "', principal mdate '" + (new Date(mdate)) + "'");
          }
          if (((_ref3 = keytab[principal]) != null ? _ref3.kvno : void 0) === kvno && keytab[principal].mdate === mdate) {
            return do_chown();
          }
          return do_ktremove();
        });
      });
    };
    do_ktremove = function() {
      return execute({
        cmd: misc.kadmin(options, "ktremove -k " + options.keytab + " " + options.principal),
        ssh: options.ssh,
        log: options.log,
        stdout: options.stdout,
        stderr: options.stderr
      }, function(err, exists, stdout, stderr) {
        if (err) {
          return next(err);
        }
        return do_ktadd();
      });
    };
    do_ktadd = function() {
      return execute({
        cmd: misc.kadmin(options, "ktadd -k " + options.keytab + " " + options.principal),
        ssh: options.ssh,
        log: options.log,
        stdout: options.stdout,
        stderr: options.stderr
      }, function(err, ktadded) {
        if (err) {
          return next(err);
        }
        modified = true;
        return do_chown();
      });
    };
    do_chown = function() {
      if (!options.keytab || (!options.uid && !options.gid)) {
        return do_chmod();
      }
      return chown({
        ssh: options.ssh,
        log: options.log,
        destination: options.keytab,
        uid: options.uid,
        gid: options.gid
      }, function(err, chowned) {
        if (err) {
          return next(err);
        }
        if (chowned) {
          modified = chowned;
        }
        return do_chmod();
      });
    };
    do_chmod = function() {
      if (!options.keytab || !options.mode) {
        return do_end();
      }
      return chmod({
        ssh: options.ssh,
        log: options.log,
        destination: options.keytab,
        mode: options.mode
      }, function(err, chmoded) {
        if (err) {
          return next(err);
        }
        if (chmoded) {
          modified = chmoded;
        }
        return do_end();
      });
    };
    do_end = function() {
      return next(null, modified);
    };
    return do_get();
  });
};

misc = require('./misc');

wrap = require('./misc/wrap');

execute = require('./execute');

chmod = require('./chmod');

chown = require('./chown');
